@@ -44,7 +44,6 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <poll.h>
-#include <unistd.h>
 #include <fcntl.h>
 
 #include <corosync/list.h>
@@ -178,7 +177,7 @@ static void delivery_callback (
 	log_pt = malloc (sizeof(log_entry_t));
 	list_init (&log_pt->list);
 
-	snprintf (log_pt->log, LOG_STR_SIZE, "%d:%d:%d:%s;",
+	snprintf (log_pt->log, LOG_STR_SIZE, "%u:%d:%d:%s;",
 		msg_pt->nodeid, msg_pt->seq, my_seq,
 		err_status_string (status_buf, 20, status));
 	list_add_tail (&log_pt->list, &msg_log_head);
@@ -212,7 +211,7 @@ static void config_change_callback (
 		if (record_config_events_g > 0) {
 			log_pt = malloc (sizeof(log_entry_t));
 			list_init (&log_pt->list);
-			snprintf (log_pt->log, LOG_STR_SIZE, "%s,%d,%d,left",
+			snprintf (log_pt->log, LOG_STR_SIZE, "%s,%u,%u,left",
 				groupName->value, left_list[i].nodeid,left_list[i].pid);
 			list_add_tail(&log_pt->list, &config_chg_log_head);
 			qb_log (LOG_INFO, "cpg event %s", log_pt->log);
@@ -222,7 +221,7 @@ static void config_change_callback (
 		if (record_config_events_g > 0) {
 			log_pt = malloc (sizeof(log_entry_t));
 			list_init (&log_pt->list);
-			snprintf (log_pt->log, LOG_STR_SIZE, "%s,%d,%d,join",
+			snprintf (log_pt->log, LOG_STR_SIZE, "%s,%u,%u,join",
 				groupName->value, joined_list[i].nodeid,joined_list[i].pid);
 			list_add_tail (&log_pt->list, &config_chg_log_head);
 			qb_log (LOG_INFO, "cpg event %s", log_pt->log);
@@ -239,7 +238,7 @@ static void my_shutdown_callback (corosync_cfg_handle_t handle,
 	corosync_cfg_shutdown_flags_t flags)
 {
 	qb_log (LOG_CRIT, "flags:%d", flags);
-	if (flags & COROSYNC_CFG_SHUTDOWN_FLAG_REQUEST) {
+	if (flags == COROSYNC_CFG_SHUTDOWN_FLAG_REQUEST) {
 		corosync_cfg_replyto_shutdown (cfg_handle, COROSYNC_CFG_SHUTDOWN_FLAG_YES);
 	}
 }
@@ -262,12 +261,16 @@ static void record_messages (void)
 static void record_config_events (int sock)
 {
 	char response[100];
+	ssize_t rc;
+	size_t send_len;
 
 	record_config_events_g = 1;
 	qb_log (LOG_INFO, "record:%d", record_config_events_g);
 
 	snprintf (response, 100, "%s", OK_STR);
-	send (sock, response, strlen (response), 0);
+	send_len = strlen (response);
+	rc = send (sock, response, send_len, 0);
+	assert(rc == send_len);
 }
 
 static void read_config_event (int sock)
@@ -275,16 +278,21 @@ static void read_config_event (int sock)
 	const char *empty = "None";
 	struct list_head * list = config_chg_log_head.next;
 	log_entry_t *entry;
+	ssize_t rc;
+	size_t send_len;
 
 	if (list != &config_chg_log_head) {
 		entry = list_entry (list, log_entry_t, list);
-		send (sock, entry->log,	strlen (entry->log), 0);
+		send_len = strlen (entry->log);
+		rc = send (sock, entry->log, send_len, 0);
 		list_del (&entry->list);
 		free (entry);
 	} else {
 		qb_log (LOG_DEBUG, "no events in list");
-		send (sock, empty, strlen (empty), 0);
+		send_len = strlen (empty);
+		rc = send (sock, empty, send_len, 0);
 	}
+	assert(rc == send_len);
 }
 
 static void read_messages (int sock, char* atmost_str)
@@ -325,7 +333,7 @@ static void read_messages (int sock, char* atmost_str)
 		}
 	}
 	rc = send (sock, big_and_buf, strlen (big_and_buf), 0);
-	assert(rc = strlen (big_and_buf));
+	assert(rc == strlen (big_and_buf));
 }
 
 static qb_loop_timer_handle more_messages_timer_handle;
@@ -529,6 +537,8 @@ static void context_test (int sock)
 {
 	char response[100];
 	char *cmp;
+	ssize_t rc;
+	size_t send_len;
 
 	cpg_context_set (cpg_handle, response);
 	cpg_context_get (cpg_handle, (void**)&cmp);
@@ -538,7 +548,9 @@ static void context_test (int sock)
 	else {
 		snprintf (response, 100, "%s", OK_STR);
 	}
-	send (sock, response, strlen (response), 0);
+	send_len = strlen (response);
+	rc = send (sock, response, send_len, 0);
+	assert(rc == send_len);
 }
 
 static void msg_blaster_zcb (int sock, char* num_to_send_str)
@@ -614,6 +626,8 @@ static void do_command (int sock, char* func, char*args[], int num_args)
 	int result;
 	char response[100];
 	struct cpg_name group_name;
+	ssize_t rc;
+	size_t send_len;
 
 	qb_log (LOG_TRACE, "RPC:%s() called.", func);
 
@@ -628,7 +642,10 @@ static void do_command (int sock, char* func, char*args[], int num_args)
 		cpg_mcast_joined (cpg_handle, CPG_TYPE_AGREED, iov, num_args);
 
 	} else if (strcmp ("cpg_join",func) == 0) {
-
+		if (strlen(args[0]) >= CPG_MAX_NAME_LENGTH) {
+			qb_log (LOG_ERR, "Invalid group name");
+			exit (1);
+		}
 		strcpy (group_name.value, args[0]);
 		group_name.length = strlen(args[0]);
 		result = cpg_join (cpg_handle, &group_name);
@@ -681,7 +698,9 @@ static void do_command (int sock, char* func, char*args[], int num_args)
 
 		cpg_local_get (cpg_handle, &local_nodeid);
 		snprintf (response, 100, "%u",local_nodeid);
-		send (sock, response, strlen (response), 0);
+		send_len = strlen (response);
+		rc = send (sock, response, send_len, 0);
+		assert(rc == send_len);
 	} else if (strcmp ("cpg_finalize", func) == 0) {
 
 		if (cpg_handle > 0) {
@@ -707,8 +726,9 @@ static void do_command (int sock, char* func, char*args[], int num_args)
 		context_test (sock);
 	} else if (strcmp ("are_you_ok_dude", func) == 0) {
 		snprintf (response, 100, "%s", OK_STR);
-		send (sock, response, strlen (response), 0);
-
+		send_len = strlen (response);
+		rc = send (sock, response, strlen (response), 0);
+		assert(rc == send_len);
 	} else if (strcmp ("cfg_shutdown", func) == 0) {
 
 		qb_log (LOG_INFO, "calling %s() called!", func);
