@@ -39,7 +39,6 @@
 #include <fcntl.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <unistd.h>
 #include <poll.h>
 #include <assert.h>
 
@@ -362,11 +361,12 @@ static void cmap_sync_init (
 
 	cmap_sync_trans_list_entries = trans_list_entries;
 	cmap_sync_member_list_entries = member_list_entries;
-	cmap_highest_config_version_received = 0;
 
 	if (icmap_get_uint64("totem.config_version", &cmap_my_config_version) != CS_OK) {
 		cmap_my_config_version = 0;
 	}
+
+	cmap_highest_config_version_received = cmap_my_config_version;
 }
 
 static int cmap_sync_process (void)
@@ -402,13 +402,6 @@ static void cmap_sync_activate (void)
 		return ;
 	}
 
-	if (cmap_highest_config_version_received == 0) {
-		log_printf(LOGSYS_LEVEL_DEBUG, "Other nodes version is 0 -> no action");
-
-		return ;
-	}
-
-
 	if (cmap_highest_config_version_received != cmap_my_config_version) {
 		log_printf(LOGSYS_LEVEL_ERROR,
 		    "Received config version (%"PRIu64") is different than my config version (%"PRIu64")! Exiting",
@@ -836,7 +829,7 @@ static cs_error_t cmap_mcast_send(enum cmap_mcast_reason reason, int argc, char
 		if (item == NULL) {
 			goto free_mem;
 		}
-		memset(item, 0, sizeof(item_len));
+		memset(item, 0, item_len);
 
 		item->value_type = value_type;
 		item->value_len = value_len;
