@@ -81,7 +81,7 @@ static uint8_t two_node = 0;
 static uint8_t wait_for_all = 0;
 static uint8_t wait_for_all_status = 0;
 
-static enum {ATB_NONE, ATB_LOWEST, ATB_HIGHEST, ATB_LIST} auto_tie_breaker = ATB_NONE;
+static enum {ATB_NONE, ATB_LOWEST, ATB_HIGHEST, ATB_LIST} auto_tie_breaker = ATB_NONE, initial_auto_tie_breaker = ATB_NONE;
 static int lowest_node_id = -1;
 static int highest_node_id = -1;
 
@@ -931,11 +931,7 @@ static int calculate_quorum(int allow_decrease, unsigned int max_expected, unsig
 			   node->node_id, node->state, node->votes, node->expected_votes);
 
 		if (node->state == NODESTATE_MEMBER) {
-			if (max_expected) {
-				node->expected_votes = max_expected;
-			} else {
-				highest_expected = max(highest_expected, node->expected_votes);
-			}
+			highest_expected = max(highest_expected, node->expected_votes);
 			total_votes += node->votes;
 			total_nodes++;
 		}
@@ -986,6 +982,22 @@ static int calculate_quorum(int allow_decrease, unsigned int max_expected, unsig
 	return newquorum;
 }
 
+static void update_node_expected_votes(int new_expected_votes)
+{
+	struct list_head *nodelist;
+	struct cluster_node *node;
+
+	if (new_expected_votes) {
+		list_iterate(nodelist, &cluster_members_list) {
+			node = list_entry(nodelist, struct cluster_node, list);
+
+			if (node->state == NODESTATE_MEMBER) {
+				node->expected_votes = new_expected_votes;
+			}
+		}
+	}
+}
+
 static void are_we_quorate(unsigned int total_votes)
 {
 	int quorate;
@@ -1126,6 +1138,8 @@ static void recalculate_quorum(int allow_decrease, int by_current_nodes)
 	}
 
 	quorum = calculate_quorum(allow_decrease, cluster_members, &total_votes);
+	update_node_expected_votes(cluster_members);
+
 	are_we_quorate(total_votes);
 
 	LEAVE();
@@ -1246,6 +1260,13 @@ static char *votequorum_readconfig(int runtime)
 	if (runtime) {
 		two_node = 0;
 		expected_votes = 0;
+		/* auto_tie_breaker cannot be changed by config reload, but
+		 * we automatically disable it on odd-sized clusters without
+		 * wait_for_all.
+		 * We may need to re-enable it when membership changes to ensure
+		 * that auto_tie_breaker is consistent across all nodes */
+		auto_tie_breaker = initial_auto_tie_breaker;
+		icmap_set_uint32("runtime.votequorum.atb_type", auto_tie_breaker);
 	}
 
 	/*
@@ -1324,6 +1345,7 @@ static char *votequorum_readconfig(int runtime)
 			parse_atb_string(atb_string);
 		}
 		free(atb_string);
+		initial_auto_tie_breaker = auto_tie_breaker;
 
 		/* allow_downscale requires ev_tracking */
 		if (allow_downscale) {
@@ -2144,7 +2166,6 @@ static void message_handler_req_exec_votequorum_reconfigure (
 {
 	const struct req_exec_quorum_reconfigure *req_exec_quorum_reconfigure = message;
 	struct cluster_node *node;
-	struct list_head *nodelist;
 
 	ENTER();
 
@@ -2154,12 +2175,8 @@ static void message_handler_req_exec_votequorum_reconfigure (
 	switch(req_exec_quorum_reconfigure->param)
 	{
 	case VOTEQUORUM_RECONFIG_PARAM_EXPECTED_VOTES:
-		list_iterate(nodelist, &cluster_members_list) {
-			node = list_entry(nodelist, struct cluster_node, list);
-			if (node->state == NODESTATE_MEMBER) {
-				node->expected_votes = req_exec_quorum_reconfigure->value;
-			}
-		}
+		update_node_expected_votes(req_exec_quorum_reconfigure->value);
+
 		votequorum_exec_send_expectedvotes_notification();
 		update_ev_barrier(req_exec_quorum_reconfigure->value);
 		if (ev_tracking) {
@@ -2651,6 +2668,7 @@ static void message_handler_req_lib_votequorum_setexpected (void *conn, const vo
 		error = CS_ERR_INVALID_PARAM;
 		goto error_exit;
 	}
+	update_node_expected_votes(req_lib_votequorum_setexpected->expected_votes);
 
 	votequorum_exec_send_reconfigure(VOTEQUORUM_RECONFIG_PARAM_EXPECTED_VOTES, us->node_id,
 					 req_lib_votequorum_setexpected->expected_votes);
