@@ -76,26 +76,45 @@ static void votequorum_expectedvotes_notification_fn(
 }
 
 
-static void votequorum_notification_fn(
+static void votequorum_quorum_notification_fn(
 	votequorum_handle_t handle,
 	uint64_t context,
 	uint32_t quorate,
-	votequorum_ring_id_t ring_id,
 	uint32_t node_list_entries,
 	votequorum_node_t node_list[]
 	)
 {
 	int i;
 
-	printf("votequorum notification called \n");
-	printf("  quorate         = %d\n", quorate);
+	printf("votequorum quorum notification called \n");
 	printf("  number of nodes = %d\n", node_list_entries);
-	printf("  current ringid  = (%u.%"PRIu64")\n", ring_id.nodeid, ring_id.seq);
+	printf("  quorate         = %d\n", quorate);
 
 	for (i = 0; i< node_list_entries; i++) {
 		printf("      %d: %s\n", node_list[i].nodeid, node_state(node_list[i].state));
 	}
-	printf("\n");
+
+}
+
+static void votequorum_nodelist_notification_fn(
+	votequorum_handle_t handle,
+	uint64_t context,
+	votequorum_ring_id_t ring_id,
+	uint32_t node_list_entries,
+	uint32_t node_list[]
+	)
+{
+	int i;
+
+	printf("votequorum nodelist notification called \n");
+	printf("  number of nodes = %d\n", node_list_entries);
+	printf("  current ringid  = (%u.%"PRIu64")\n", ring_id.nodeid, ring_id.seq);
+	printf("  nodes: ");
+
+	for (i = 0; i< node_list_entries; i++) {
+		printf("%d ", node_list[i]);
+	}
+	printf("\n\n");
 }
 
 
@@ -110,7 +129,8 @@ int main(int argc, char *argv[])
 		return 0;
 	}
 
-	callbacks.votequorum_notify_fn = votequorum_notification_fn;
+	callbacks.votequorum_quorum_notify_fn = votequorum_quorum_notification_fn;
+	callbacks.votequorum_nodelist_notify_fn = votequorum_nodelist_notification_fn;
 	callbacks.votequorum_expectedvotes_notify_fn = votequorum_expectedvotes_notification_fn;
 
 	if ( (err=votequorum_initialize(&g_handle, &callbacks)) != CS_OK)
