@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009-2012 Red Hat, Inc.
+ * Copyright (c) 2009-2014 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -36,6 +36,7 @@
 #include <config.h>
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -68,6 +69,12 @@ typedef enum {
 	CMD_UNREGISTER_QDEVICE
 } command_t;
 
+typedef enum {
+	SORT_ADDR,
+	SORT_NODEID,
+	SORT_NODENAME
+} sorttype_t;
+
 /*
  * global vars
  */
@@ -96,19 +103,37 @@ static quorum_callbacks_t q_callbacks = {
 /*
  * quorum call back vars
  */
+
+/* Containing struct to keep votequorum & normal quorum bits together */
+typedef struct {
+      struct votequorum_info *vq_info; /* Might be NULL if votequorum not present */
+      char *name;  /* Might be IP address or NULL */
+      int node_id; /* Always present */
+} view_list_entry_t;
+
+static view_list_entry_t *g_view_list;
 static uint32_t g_quorate;
 static uint64_t g_ring_id;
+static uint32_t g_ring_id_rep_node;
 static uint32_t g_view_list_entries;
-static uint32_t *g_view_list = NULL;
 static uint32_t g_called;
+static uint32_t g_vq_called;
+static uint32_t g_show_all_addrs = 0;
 
 /*
  * votequorum bits
  */
+static void votequorum_notification_fn(
+	votequorum_handle_t handle,
+	uint64_t context,
+	votequorum_ring_id_t ring_id,
+	uint32_t node_list_entries,
+	uint32_t node_list[]);
 static votequorum_handle_t v_handle;
 static votequorum_callbacks_t v_callbacks = {
-	.votequorum_notify_fn = NULL,
-	.votequorum_expectedvotes_notify_fn = NULL
+	.votequorum_quorum_notify_fn = NULL,
+	.votequorum_expectedvotes_notify_fn = NULL,
+	.votequorum_nodelist_notify_fn = votequorum_notification_fn,
 };
 static uint32_t our_nodeid = 0;
 
@@ -133,14 +158,17 @@ static void show_usage(const char *name)
 	printf("  options:\n");
 	printf("\n");
 	printf("  -s             show quorum status\n");
-	printf("  -m             monitor quorum status\n");
+	printf("  -m             constantly monitor quorum status\n");
 	printf("  -l             list nodes\n");
+	printf("  -a             show all names or addresses for each node\n");
 	printf("  -p             when used with -s or -l, generates machine parsable output\n");
 	printf("  -v <votes>     change the number of votes for a node (*)\n");
-	printf("  -n <nodeid>    optional nodeid of node for -v (*)\n");
+	printf("  -n <nodeid>    optional nodeid of node for -v\n");
 	printf("  -e <expected>  change expected votes for the cluster (*)\n");
 	printf("  -H             show nodeids in hexadecimal rather than decimal\n");
 	printf("  -i             show node IP addresses instead of the resolved name\n");
+	printf("  -p             when used with -s or -l, generates machine parsable output\n");
+	printf("  -o <a|n|i>     order by [a] IP address (default), [n] name, [i] nodeid\n");
 	printf("  -f             forcefully unregister a quorum device *DANGEROUS* (*)\n");
 	printf("  -h             show this help text\n");
 	printf("  -V             show version and exit\n");
@@ -288,18 +316,25 @@ static const char *node_name(uint32_t nodeid, name_format_t name_format)
 	int err;
 	int numaddrs;
 	corosync_cfg_node_address_t addrs[INTERFACE_MAX];
-	static char buf[INET6_ADDRSTRLEN];
+	static char buf[(INET6_ADDRSTRLEN + 1) * INTERFACE_MAX];
 	const char *nodelist_name = NULL;
 	socklen_t addrlen;
 	struct sockaddr_storage *ss;
+	int start_addr = 0;
+	int i;
+	int bufptr = 0;
 
+	buf[0] = '\0';
+
+	/* If a name is required, always look for the nodelist node0_addr name first */
 	if (name_format == ADDRESS_FORMAT_NAME) {
 		nodelist_name = node_name_by_nodelist(nodeid);
 	}
-
 	if ((nodelist_name) &&
 	    (strlen(nodelist_name) > 0)) {
-		return nodelist_name;
+		start_addr = 1;
+		strcpy(buf, nodelist_name);
+		bufptr = strlen(buf);
 	}
 
 	err = corosync_cfg_get_node_addrs(c_handle, nodeid, INTERFACE_MAX, &numaddrs, addrs);
@@ -308,23 +343,48 @@ static const char *node_name(uint32_t nodeid, name_format_t name_format)
 		return "";
 	}
 
-	ss = (struct sockaddr_storage *)addrs[0].address;
-
-	if (ss->ss_family == AF_INET6) {
-		addrlen = sizeof(struct sockaddr_in6);
-	} else {
-		addrlen = sizeof(struct sockaddr_in);
+	/* Don't show all addressess */
+	if (!g_show_all_addrs) {
+		numaddrs = 1;
 	}
 
-	if (!getnameinfo(
-		(struct sockaddr *)addrs[0].address, addrlen,
-		buf, sizeof(buf),
-		NULL, 0,
-		(name_format == ADDRESS_FORMAT_IP)?NI_NUMERICHOST:0)) {
-			return buf;
+	for (i=start_addr; i<numaddrs; i++) {
+
+		if (i) {
+			buf[bufptr++] = ',';
+			buf[bufptr++] = ' ';
+		}
+
+		ss = (struct sockaddr_storage *)addrs[i].address;
+
+		if (ss->ss_family == AF_INET6) {
+			addrlen = sizeof(struct sockaddr_in6);
+		} else {
+			addrlen = sizeof(struct sockaddr_in);
+		}
+
+		if (!getnameinfo(
+			    (struct sockaddr *)addrs[i].address, addrlen,
+			    buf+bufptr, sizeof(buf)-bufptr,
+			    NULL, 0,
+			    (name_format == ADDRESS_FORMAT_IP)?NI_NUMERICHOST:0)) {
+			bufptr += strlen(buf+bufptr);
+		}
 	}
 
-	return "";
+	return buf;
+}
+
+
+static void votequorum_notification_fn(
+	votequorum_handle_t handle,
+	uint64_t context,
+	votequorum_ring_id_t ring_id,
+	uint32_t node_list_entries,
+	uint32_t node_list[])
+{
+	g_ring_id_rep_node = ring_id.nodeid;
+	g_vq_called = 1;
 }
 
 static void quorum_notification_fn(
@@ -334,6 +394,8 @@ static void quorum_notification_fn(
 	uint32_t view_list_entries,
 	uint32_t *view_list)
 {
+        int i;
+
 	g_called = 1;
 	g_quorate = quorate;
 	g_ring_id = ring_id;
@@ -341,9 +403,13 @@ static void quorum_notification_fn(
 	if (g_view_list) {
 		free(g_view_list);
 	}
-	g_view_list = malloc(sizeof(uint32_t) * view_list_entries);
+	g_view_list = malloc(sizeof(view_list_entry_t) * view_list_entries);
 	if (g_view_list) {
-		memcpy(g_view_list, view_list,sizeof(uint32_t) * view_list_entries);
+	        for (i=0; i< view_list_entries; i++) {
+		        g_view_list[i].node_id = view_list[i];
+			g_view_list[i].name = NULL;
+			g_view_list[i].vq_info = NULL;
+		}
 	}
 }
 
@@ -368,25 +434,56 @@ static void print_uint32_padded(uint32_t value)
 	print_string_padded(buf);
 }
 
-static void display_nodes_data(nodeid_format_t nodeid_format, name_format_t name_format)
+/* for qsort */
+static int compare_nodeids(const void *one, const void *two)
+{
+      const view_list_entry_t *info1 = one;
+      const view_list_entry_t *info2 = two;
+
+      if (info1->node_id == info2->node_id) {
+	  return 0;
+      }
+      if (info1->node_id > info2->node_id) {
+	  return 1;
+      }
+      return -1;
+}
+
+static int compare_nodenames(const void *one, const void *two)
+{
+      const view_list_entry_t *info1 = one;
+      const view_list_entry_t *info2 = two;
+
+      return strcmp(info1->name, info2->name);
+}
+
+static void display_nodes_data(nodeid_format_t nodeid_format, name_format_t name_format, sorttype_t sort_type)
 {
 	int i, display_qdevice = 0;
+	unsigned int our_flags = 0;
 	struct votequorum_info info[g_view_list_entries];
-
 	/*
 	 * cache node info because we need to parse them twice
 	 */
 	if (v_handle) {
 		for (i=0; i < g_view_list_entries; i++) {
-			if (votequorum_getinfo(v_handle, g_view_list[i], &info[i]) != CS_OK) {
-				printf("Unable to get node %u info\n", g_view_list[i]);
+			if (votequorum_getinfo(v_handle, g_view_list[i].node_id, &info[i]) != CS_OK) {
+				printf("Unable to get node %u info\n", g_view_list[i].node_id);
 			}
+			g_view_list[i].vq_info = &info[i];
 			if (info[i].flags & VOTEQUORUM_INFO_QDEVICE_REGISTERED) {
 				display_qdevice = 1;
 			}
 		}
 	}
 
+	/*
+	 * Get node names
+	 */
+	for (i=0; i < g_view_list_entries; i++) {
+	        g_view_list[i].name = strdup(node_name(g_view_list[i].node_id, name_format));
+	}
+
 	printf("\nMembership information\n");
 	printf("----------------------\n");
 
@@ -399,11 +496,18 @@ static void display_nodes_data(nodeid_format_t nodeid_format, name_format_t name
 	}
 	printf("Name\n");
 
+	/* corosync sends them already sorted by address */
+	if (sort_type == SORT_NODEID) {
+		qsort(g_view_list, g_view_list_entries, sizeof(view_list_entry_t), compare_nodeids);
+	}
+	if (sort_type == SORT_NODENAME) {
+		qsort(g_view_list, g_view_list_entries, sizeof(view_list_entry_t), compare_nodenames);
+	}
 	for (i=0; i < g_view_list_entries; i++) {
 		if (nodeid_format == NODEID_FORMAT_DECIMAL) {
-			print_uint32_padded(g_view_list[i]);
+			print_uint32_padded(g_view_list[i].node_id);
 		} else {
-			printf("0x%08x ", g_view_list[i]);
+			printf("0x%08x ", g_view_list[i].node_id);
 		}
 		if (v_handle) {
 			int votes = -1;
@@ -426,14 +530,18 @@ static void display_nodes_data(nodeid_format_t nodeid_format, name_format_t name
 				}
 			}
 		}
-		printf("%s", node_name(g_view_list[i], name_format));
-		if (g_view_list[i] == our_nodeid) {
+		printf("%s", g_view_list[i].name);
+		if (g_view_list[i].node_id == our_nodeid) {
 			printf(" (local)");
+			our_flags = info[i].flags;
 		}
 		printf("\n");
 	}
 
 	if (g_view_list_entries) {
+	        for (i=0; i < g_view_list_entries; i++) {
+		        free(g_view_list[i].name);
+		}
 		free(g_view_list);
 		g_view_list = NULL;
 	}
@@ -444,14 +552,27 @@ static void display_nodes_data(nodeid_format_t nodeid_format, name_format_t name
 		} else {
 			printf("0x%08x ", VOTEQUORUM_QDEVICE_NODEID);
 		}
-		print_uint32_padded(info[0].qdevice_votes);
-		printf("           %s\n", info[0].qdevice_name);
+		/* If the quorum device is inactive on this node then show votes as 0
+		   so that the display is not confusing */
+		if (our_flags & VOTEQUORUM_INFO_QDEVICE_CAST_VOTE) {
+			print_uint32_padded(info[0].qdevice_votes);
+		}
+		else {
+			print_uint32_padded(0);
+		}
+		printf("           %s", info[0].qdevice_name);
+		if (our_flags & VOTEQUORUM_INFO_QDEVICE_CAST_VOTE) {
+			printf("\n");
+		}
+		else {
+			printf(" (votes %d)\n", info[0].qdevice_votes);
+		}
 	}
 
 }
 
 static int display_quorum_data(int is_quorate,
-			       nodeid_format_t nodeid_format, name_format_t name_format,
+			       nodeid_format_t nodeid_format, name_format_t name_format, sorttype_t sort_type,
 			       int loop)
 {
 	struct votequorum_info info;
@@ -476,7 +597,13 @@ static int display_quorum_data(int is_quorate,
 	} else {
 		printf("Node ID:          0x%08x\n", our_nodeid);
 	}
-	printf("Ring ID:          %" PRIu64 "\n", g_ring_id);
+
+	if (v_handle) {
+		printf("Ring ID:          %d/%" PRIu64 "\n", g_ring_id_rep_node, g_ring_id);
+	}
+	else {
+		printf("Ring ID:          %" PRIu64 "\n", g_ring_id);
+	}
 	printf("Quorate:          %s\n", is_quorate?"Yes":"No");
 
 	if (!v_handle) {
@@ -504,7 +631,7 @@ static int display_quorum_data(int is_quorate,
 		fprintf(stderr, "Unable to get node info: %s\n", cs_strerror(err));
 	}
 
-	display_nodes_data(nodeid_format, name_format);
+	display_nodes_data(nodeid_format, name_format, sort_type);
 
 	return err;
 }
@@ -514,7 +641,7 @@ static int display_quorum_data(int is_quorate,
  *         0 if not quorate
  *        -1 on error
  */
-static int show_status(nodeid_format_t nodeid_format, name_format_t name_format)
+static int show_status(nodeid_format_t nodeid_format, name_format_t name_format, sorttype_t sort_type)
 {
 	int is_quorate;
 	int err;
@@ -543,12 +670,28 @@ static int show_status(nodeid_format_t nodeid_format, name_format_t name_format)
 		fprintf(stderr, "Unable to stop quorum status tracking: %s\n", cs_strerror(err));
 	}
 
+	if (using_votequorum()) {
+
+		if ( (err=votequorum_trackstart(v_handle, 0LL, CS_TRACK_CURRENT)) != CS_OK) {
+			fprintf(stderr, "Unable to start votequorum status tracking: %s\n", cs_strerror(err));
+			goto quorum_err;
+		}
+
+		g_vq_called = 0;
+		while (g_vq_called == 0 && err == CS_OK) {
+			err = votequorum_dispatch(v_handle, CS_DISPATCH_ONE);
+			if (err != CS_OK) {
+				fprintf(stderr, "Unable to dispatch votequorum status: %s\n", cs_strerror(err));
+			}
+		}
+	}
+
 quorum_err:
 	if (err != CS_OK) {
 		return -1;
 	}
 
-	err = display_quorum_data(is_quorate, nodeid_format, name_format, 0);
+	err = display_quorum_data(is_quorate, nodeid_format, name_format, sort_type, 0);
 	if (err != CS_OK) {
 		return -1;
 	}
@@ -556,13 +699,13 @@ quorum_err:
 	return is_quorate;
 }
 
-static int monitor_status(nodeid_format_t nodeid_format, name_format_t name_format) {
+static int monitor_status(nodeid_format_t nodeid_format, name_format_t name_format, sorttype_t sort_type) {
 	int err;
 	int loop = 0;
 
 	if (q_type == QUORUM_FREE) {
 		printf("\nQuorum is not configured - cannot monitor\n");
-		return show_status(nodeid_format, name_format);
+		return show_status(nodeid_format, name_format, sort_type);
 	}
 
 	err=quorum_trackstart(q_handle, CS_TRACK_CHANGES);
@@ -571,13 +714,32 @@ static int monitor_status(nodeid_format_t nodeid_format, name_format_t name_form
 		goto quorum_err;
 	}
 
+	if (using_votequorum()) {
+		if ( (err=votequorum_trackstart(v_handle, 0LL, CS_TRACK_CHANGES)) != CS_OK) {
+			fprintf(stderr, "Unable to start votequorum status tracking: %s\n", cs_strerror(err));
+			goto quorum_err;
+		}
+	}
+
+
 	while (1) {
 		err = quorum_dispatch(q_handle, CS_DISPATCH_ONE);
 		if (err != CS_OK) {
 			fprintf(stderr, "Unable to dispatch quorum status: %s\n", cs_strerror(err));
 			goto quorum_err;
 		}
-		err = display_quorum_data(g_quorate, nodeid_format, name_format, loop);
+		if (using_votequorum()) {
+			g_vq_called = 0;
+			while (!g_vq_called) {
+				err = votequorum_dispatch(v_handle, CS_DISPATCH_ONE);
+				if (err != CS_OK) {
+					fprintf(stderr, "Unable to dispatch votequorum status: %s\n", cs_strerror(err));
+					goto quorum_err;
+				}
+			}
+		}
+
+		err = display_quorum_data(g_quorate, nodeid_format, name_format, sort_type, loop);
 		printf("\n");
 		loop = 1;
 		if (err != CS_OK) {
@@ -590,7 +752,7 @@ quorum_err:
 	return -1;
 }
 
-static int show_nodes(nodeid_format_t nodeid_format, name_format_t name_format)
+static int show_nodes(nodeid_format_t nodeid_format, name_format_t name_format, sorttype_t sort_type)
 {
 	int err;
 	int result = EXIT_FAILURE;
@@ -610,7 +772,7 @@ static int show_nodes(nodeid_format_t nodeid_format, name_format_t name_format)
 		}
 	}
 
-	display_nodes_data(nodeid_format, name_format);
+	display_nodes_data(nodeid_format, name_format, sort_type);
 
 	result = EXIT_SUCCESS;
 err_exit:
@@ -680,7 +842,7 @@ static int init_all(void) {
 	}
 
 	if (cmap_get_uint32(cmap_handle, "runtime.votequorum.this_node_id", &our_nodeid) != CS_OK) {
-		fprintf(stderr, "Unable to retrive this node nodeid\n");
+		fprintf(stderr, "Unable to retrieve this_node_id\n");
 		goto out;
 	}
 
@@ -705,7 +867,7 @@ static void close_all(void) {
 }
 
 int main (int argc, char *argv[]) {
-	const char *options = "VHslpmfe:v:hin:";
+	const char *options = "VHaslpmfe:v:hin:o:";
 	char *endptr;
 	int opt;
 	int votes = 0;
@@ -715,6 +877,8 @@ int main (int argc, char *argv[]) {
 	nodeid_format_t nodeid_format = NODEID_FORMAT_DECIMAL;
 	name_format_t address_format = ADDRESS_FORMAT_NAME;
 	command_t command_opt = CMD_SHOWSTATUS;
+	sorttype_t sort_opt = SORT_ADDR;
+	char sortchar;
 	long int l;
 
 	if (init_all()) {
@@ -735,6 +899,9 @@ int main (int argc, char *argv[]) {
 		case 's':
 			command_opt = CMD_SHOWSTATUS;
 			break;
+		case 'a':
+			g_show_all_addrs = 1;
+			break;
 		case 'm':
 			command_opt = CMD_MONITOR;
 			break;
@@ -787,6 +954,21 @@ int main (int argc, char *argv[]) {
 				exit(2);
 			}
 			break;
+		case 'o':
+			sortchar = optarg[0];
+			switch (sortchar) {
+			        case 'a': sort_opt = SORT_ADDR;
+					break;
+			        case 'i': sort_opt = SORT_NODEID;
+					break;
+			        case 'n': sort_opt = SORT_NODENAME;
+					break;
+			        default:
+					fprintf(stderr, "Invalid ordering option. valid orders are a(address), i(node ID) or n(name)\n");
+					exit(2);
+					break;
+			}
+			break;
 		case 'V':
 			printf("corosync-quorumtool version: %s\n", VERSION);
 			exit(0);
@@ -805,10 +987,10 @@ int main (int argc, char *argv[]) {
 		ret = -1;
 		break;
 	case CMD_SHOWNODES:
-		ret = show_nodes(nodeid_format, address_format);
+		ret = show_nodes(nodeid_format, address_format, sort_opt);
 		break;
 	case CMD_SHOWSTATUS:
-		ret = show_status(nodeid_format, address_format);
+		ret = show_status(nodeid_format, address_format, sort_opt);
 		break;
 	case CMD_SETVOTES:
 		if (!nodeid_set) {
@@ -820,7 +1002,7 @@ int main (int argc, char *argv[]) {
 		ret = set_expected(votes);
 		break;
 	case CMD_MONITOR:
-		ret = monitor_status(nodeid_format, address_format);
+		ret = monitor_status(nodeid_format, address_format, sort_opt);
 		break;
 	case CMD_UNREGISTER_QDEVICE:
 		ret = unregister_qdevice();
